@@ -43,6 +43,10 @@ struct mapping_t Menus[] = {
  { "pgp",	MENU_PGP },
 #endif  
   
+#ifdef HAVE_SMIME
+ { "smime",	MENU_SMIME },
+#endif
+ 
 #ifdef MIXMASTER
   { "mix", 	MENU_MIX },
 #endif
@@ -533,6 +537,10 @@ void km_init (void)
   create_bindings (OpPgp, MENU_PGP);
 #endif
 
+#ifdef HAVE_SMIME
+  create_bindings (OpSmime, MENU_SMIME);
+#endif
+
 #ifdef MIXMASTER
   create_bindings (OpMix, MENU_MIX);
   
@@ -730,6 +738,8 @@ struct binding_t *km_get_table (int menu)
       return OpPost;
     case MENU_FOLDER:
       return OpBrowser;
+    case MENU_ALIAS:
+      return OpAlias;
     case MENU_ATTACH:
       return OpAttach;
     case MENU_EDITOR:
