@@ -34,6 +34,10 @@
 #include "pgp.h"
 #endif
 
+#ifdef HAVE_SMIME
+#include "smime.h"
+#endif
+
 
 
 #include <ctype.h>
@@ -197,13 +201,13 @@ const char *mutt_attach_fmt (char *dest,
     case 'C':
       if (!optional)
       {
-	if (mutt_is_text_type (aptr->content->type, aptr->content->subtype) &&
+	if (mutt_is_text_part (aptr->content) &&
 	    mutt_get_body_charset (charset, sizeof (charset), aptr->content))
 	  mutt_format_s (dest, destlen, prefix, charset);
 	else
 	  mutt_format_s (dest, destlen, prefix, "");
       }
-      else if (!mutt_is_text_type (aptr->content->type, aptr->content->subtype) ||
+      else if (!mutt_is_text_part (aptr->content) ||
 	       !mutt_get_body_charset (charset, sizeof (charset), aptr->content))
         optional = 0;
       break;
@@ -354,9 +358,13 @@ void attach_entry (char *b, size_t blen, MUTTMENU *menu, int num)
   mutt_FormatString (b, blen, NONULL (AttachFormat), mutt_attach_fmt, (unsigned long) (((ATTACHPTR **)menu->data)[num]), M_FORMAT_ARROWCURSOR);
 }
 
-int mutt_tag_attach (MUTTMENU *menu, int n)
+int mutt_tag_attach (MUTTMENU *menu, int n, int m)
 {
-  return ((((ATTACHPTR **) menu->data)[n]->content->tagged = !((ATTACHPTR **) menu->data)[n]->content->tagged) ? 1 : -1);
+  BODY *cur = ((ATTACHPTR **) menu->data)[n]->content;
+  int ot = cur->tagged;
+  
+  cur->tagged = (m >= 0 ? m : !cur->tagged);
+  return cur->tagged - ot;
 }
 
 int mutt_is_message_type (int type, const char *subtype)
@@ -641,7 +649,7 @@ static void print_attachment_list (FILE *fp, int tag, BODY *top, STATE *state)
 	  FILE *ifp;
 
 	  mutt_mktemp (newfile);
-	  if (mutt_decode_save_attachment (fp, top, newfile, 0, 0) == 0)
+	  if (mutt_decode_save_attachment (fp, top, newfile, M_PRINTING, 0) == 0)
 	  {
 	    if ((ifp = fopen (newfile, "r")) != NULL)
 	    {
@@ -831,8 +839,10 @@ void mutt_view_attachments (HEADER *hdr)
 
 
 
-#ifdef HAVE_PGP
-  int pgp = 0;
+
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
+  int secured = 0;
+  int need_secured = 0;
 #endif
 
 
@@ -858,24 +868,48 @@ void mutt_view_attachments (HEADER *hdr)
 
 
 
-#ifdef HAVE_PGP
-  if((hdr->pgp & PGPENCRYPT) && !pgp_valid_passphrase())
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
+  if (hdr->security & ENCRYPT)
   {
-    mx_close_message (&msg);
-    return;
-  }
-  
-  if ((hdr->pgp & PGPENCRYPT) && mutt_is_multipart_encrypted(hdr->content))
-  {
-    if (pgp_decrypt_mime (msg->fp, &fp, hdr->content, &cur))
+    need_secured  = 1;
+    
+    if (!crypt_valid_passphrase(hdr->security))
     {
       mx_close_message (&msg);
       return;
     }
-    pgp = 1;
+#ifdef HAVE_SMIME
+    if (hdr->security & APPLICATION_SMIME)
+    {
+      if (hdr->env)
+	  smime_getkeys (hdr->env);
+
+      if (mutt_is_application_smime(hdr->content))
+	secured = ! smime_decrypt_mime (msg->fp, &fp, hdr->content, &cur);
+      else
+	need_secured = 0;
+    }
+#endif  
+#ifdef HAVE_PGP
+    if (hdr->security & APPLICATION_PGP)
+    {
+      if (mutt_is_multipart_encrypted(hdr->content))
+	secured = !pgp_decrypt_mime (msg->fp, &fp, hdr->content, &cur);
+      else
+	need_secured = 0;
+    }
+#endif
+
+    if (need_secured && !secured)
+    {
+      mx_close_message (&msg);
+      mutt_error _("Can't decrypt encrypted message!");
+      return;
+    }
   }
-  else
-#endif /* HAVE_PGP */
+  
+  if (!need_secured)
+#endif /* HAVE_SMIME || HAVVE_PGP */
   {
     fp = msg->fp;
     cur = hdr->content;
@@ -930,11 +964,14 @@ void mutt_view_attachments (HEADER *hdr)
         break;
       
 
-#ifdef HAVE_PGP
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
       case OP_FORGET_PASSPHRASE:
-        mutt_forget_passphrase ();
+        crypt_forget_passphrase ();
         break;
+#endif
       
+
+#ifdef HAVE_PGP
       case OP_EXTRACT_KEYS:
         pgp_extract_keys_from_attachment_list (fp, menu->tagprefix, 
 		  menu->tagprefix ? cur : idx[menu->current]->content);
@@ -945,7 +982,7 @@ void mutt_view_attachments (HEADER *hdr)
         if (pgp_check_traditional (fp, menu->tagprefix ? cur : idx[menu->current]->content,
 				   menu->tagprefix))
         {
-	  hdr->pgp = pgp_query (cur);
+	  hdr->security = crypt_query (cur);
 	  menu->redraw = REDRAW_FULL;
 	}
         break;
@@ -987,11 +1024,11 @@ void mutt_view_attachments (HEADER *hdr)
 
 
 
-#ifdef HAVE_PGP
-        if (hdr->pgp)
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
+        if (hdr->security)
         {
 	  mutt_message _(
-	    "Deletion of attachments from PGP messages is unsupported.");
+	    "Deletion of attachments from encrypted messages is unsupported.");
 	}
         else
 #endif
@@ -1122,13 +1159,14 @@ void mutt_view_attachments (HEADER *hdr)
 
 
 
-#ifdef HAVE_PGP
-	if (pgp)
+
+#if defined(HAVE_PGP) || defined(HAVE_SMIME)
+        if (need_secured && secured)
 	{
 	  fclose (fp);
 	  mutt_free_body (&cur);
 	}
-#endif /* HAVE_PGP */
+#endif /* HAVE_PGP || HAVE_SMIME */
 
 
 
